@@ -24,6 +24,9 @@ pretty_json <- function( json, ... ) UseMethod("pretty_json")
 #' @export
 pretty_json.json <- function( json, ... ) rcpp_pretty_json( json )
 
+#' @export
+pretty_json.character <- function( json, ... ) pretty_json( as.json( json ) )
+
 #' @export
 pretty_json.default <- function( json, ... ) {
   js <- to_json( json, ... )
@@ -51,6 +54,9 @@ minify_json <- function( json, ... ) UseMethod("minify_json")
 #' @export
 minify_json.json <- function( json, ... ) rcpp_minify_json( json ) 
 
+#' @export
+minify_json.character <- function( json, ... ) minify_json( as.json( json ) )
+
 #' @export
 minify_json.default <- function( json, ... ) to_json( json, ... )
 
