@@ -46,22 +46,22 @@ test_that("data.frame - all R data types are converted", {
 test_that("data.frame - complex columns are jsonified", {
   df <- data.frame( id = 1, val = I(list(c(0))))
   js <- to_json( df )
-  expect_equal( as.character( js ), "[{\"id\":1.0,\"val\":[[0.0]]}]")
+  expect_equal( as.character( js ), "[{\"id\":1.0,\"val\":[0.0]}]")
   expect_true( jsonify::validate_json( js ) ) 
   
   df <- data.frame( id = 1, val = I(list(c(0))))
   js <- to_json( df, unbox = TRUE  )
-  expect_equal( as.character( js ), "[{\"id\":1.0,\"val\":[0.0]}]")
+  expect_equal( as.character( js ), "[{\"id\":1.0,\"val\":0.0}]")
   expect_true( jsonify::validate_json( js ) ) 
   
   df <- data.frame( id = 1, val = I(list(c(0,0))))
   js <- to_json( df )
-  expect_equal( as.character( js ), "[{\"id\":1.0,\"val\":[[0.0,0.0]]}]")
+  expect_equal( as.character( js ), "[{\"id\":1.0,\"val\":[0.0,0.0]}]")
   expect_true( jsonify::validate_json( js ) ) 
   
   df <- data.frame( id = 1, val = I(list(letters[1:5])))
   js <- to_json( df )
-  expect_equal( as.character( js ), "[{\"id\":1.0,\"val\":[[\"a\",\"b\",\"c\",\"d\",\"e\"]]}]")
+  expect_equal( as.character( js ), "[{\"id\":1.0,\"val\":[\"a\",\"b\",\"c\",\"d\",\"e\"]}]")
   expect_true( jsonify::validate_json( js ) ) 
   
   df <- data.frame(
@@ -69,11 +69,14 @@ test_that("data.frame - complex columns are jsonified", {
     , norm = c(1.1, 2.2, 3.3)
     , letters = c("a","x","B")
     , val = I(list(c(0,0),1,list(c(2,3), myname = c(4,5), c("a"))))
-   , stringsAsFactors = F)
+   , stringsAsFactors = F
+   )
+  
   js <- to_json( df ) 
+  
   expect_equal(
     as.character( js ) , 
-    '[{"id":1,"norm":1.1,"letters":"a","val":[[0.0,0.0]]},{"id":2,"norm":2.2,"letters":"x","val":[[1.0]]},{"id":3,"norm":3.3,"letters":"B","val":[{"1":[2.0,3.0],"myname":[4.0,5.0],"3":["a"]}]}]'
+    '[{"id":1,"norm":1.1,"letters":"a","val":[0.0,0.0]},{"id":2,"norm":2.2,"letters":"x","val":[1.0]},{"id":3,"norm":3.3,"letters":"B","val":{"":[2.0,3.0],"myname":[4.0,5.0],"":["a"]}}]'
   )
   expect_true( jsonify::validate_json( js ) ) 
   
@@ -83,11 +86,13 @@ test_that("data.frame - complex columns are jsonified", {
     , norm = c(1.1, 2.2, 3.3)
     , letters = c("a","x","B")
     , val = I(list(c(0),1,list(c(2), myname = c(4,5), c("a"))))
-    , stringsAsFactors = F)
+    , stringsAsFactors = F
+    )
+  
   js <- to_json( df, unbox = TRUE )
   expect_equal(
-    as.character( js ) , 
-    '[{"id":1,"norm":1.1,"letters":"a","val":[0.0]},{"id":2,"norm":2.2,"letters":"x","val":[1.0]},{"id":3,"norm":3.3,"letters":"B","val":[{"1":2.0,"myname":[4.0,5.0],"3":"a"}]}]'
+    as.character( js ), 
+    '[{"id":1,"norm":1.1,"letters":"a","val":0.0},{"id":2,"norm":2.2,"letters":"x","val":1.0},{"id":3,"norm":3.3,"letters":"B","val":{"":2.0,"myname":[4.0,5.0],"":"a"}}]'
   )
   expect_true( validate_json( js ) )
   
@@ -97,10 +102,12 @@ test_that("data.frame - complex columns are jsonified", {
     , val2 = I(list( a = "a", b = c("b","c") ) )
     , val3 = I(list( l = list( 1:3, l2 = c("a","b")), 1))
     )
+  
   js <- to_json( df )
+  
   expect_equal(
     as.character( js ),
-    '[{"id":1,"val":{"x":[1,2]},"val2":{"a":["a"]},"val3":{"l":{"1":[1,2,3],"l2":["a","b"]}}},{"id":2,"val":{"y":[3,4,5,6]},"val2":{"b":["b","c"]},"val3":{"1":[1.0]}}]'
+    '[{"id":1,"val":{"x":[1,2]},"val2":{"a":["a"]},"val3":{"l":{"":[1,2,3],"l2":["a","b"]}}},{"id":2,"val":{"y":[3,4,5,6]},"val2":{"b":["b","c"]},"val3":{"":[1.0]}}]'
   )
   expect_true( validate_json( js ) )
 })
@@ -129,21 +136,21 @@ test_that("Nulls, NAs and Infs work",{
   expect_equal(to_json(data.frame(x = c(1,2,NULL,4))), to_json(data.frame(x = c(1,2,4))))
 })
 
-test_that("roundtrips with jsonlite work", {
+test_that("roundtrips work", {
   df <- data.frame(x = c("a", NA_character_), stringsAsFactors = F)
-  expect_equal( jsonlite::fromJSON( to_json( df ) ), df )
+  expect_equal( from_json( to_json( df ) ), df )
   df <- data.frame(x = c(1, NA_integer_), stringsAsFactors = F)
-  expect_equal( jsonlite::fromJSON( to_json( df ) ), df )
+  expect_equal( from_json( to_json( df ) ), df )
   df <- data.frame(x = c(1.0, NA_real_), stringsAsFactors = F)
-  expect_equal( jsonlite::fromJSON( to_json( df ) ), df )
+  expect_equal( from_json( to_json( df ) ), df )
   df <- data.frame(x = c("a", NULL), stringsAsFactors = F)
-  expect_equal( jsonlite::fromJSON( to_json( df ) ), df )
+  expect_equal( from_json( to_json( df ) ), df )
   df <- data.frame(x = c(T, NA), stringsAsFactors = F)
-  expect_equal( jsonlite::fromJSON( to_json( df ) ), df )
-  df <- data.frame(x = c(1, Inf), stringsAsFactors = F)
-  expect_equal( jsonlite::fromJSON( to_json( df ) ), df )
-  df <- data.frame(x = c(1, -Inf), stringsAsFactors = F)
-  expect_equal( jsonlite::fromJSON( to_json( df ) ), df )
+  expect_equal( from_json( to_json( df ) ), df )
+  # df <- data.frame(x = c(1, Inf), stringsAsFactors = F)
+  # expect_equal( from_json( to_json( df ) ), df )
+  # df <- data.frame(x = c(1, -Inf), stringsAsFactors = F)
+  # expect_equal( from_json( to_json( df ) ), df )
 })
 
 
@@ -185,4 +192,35 @@ test_that("ISSUE 38 - data.frames inside data.frames works", {
   expect_true( validate_json( js ) )
   expect_equal( as.character(js) , '[{"id":1.0,"details.val1":"a","details.val2":"b","details.val3":"c"},{"id":1.0,"details.val1":"b","details.val2":"c","details.val3":"d"}]')
   
-})
\ No newline at end of file
+})
+
+test_that("data.frame with a matrix-column works", {
+  
+  df <- data.frame( id = 1:2, mat = I(matrix(1:4, ncol = 2)))
+  js <- to_json( df )
+  expect_equal( as.character(js), '[{"id":1,"mat":[1,3]},{"id":2,"mat":[2,4]}]')
+  expect_true( validate_json( js ) )
+  
+  js <- to_json( df, by = "col" )
+  expect_equal( as.character( js ), '{"id":[1,2],"mat":[[1,2],[3,4]]}')
+
+  ## Issue 42
+  df <- structure(list(fill_colour = structure(c(68, 49, 53, 253, 1, 
+  104, 183, 231, 84, 142, 121, 37, 255, 255, 255, 255), .Dim = c(4L, 
+  4L)), geometry = c(1, 2, -5, 0.3), lat = 1:4, lon = c(1, 2, -5, 
+  0.3)), class = "data.frame", row.names = c(NA, 4L))
+  
+  js <- to_json( df, by = "row" )
+  expect_equal( as.character( js ), '[{"fill_colour":[68.0,1.0,84.0,255.0],"geometry":1.0,"lat":1,"lon":1.0},{"fill_colour":[49.0,104.0,142.0,255.0],"geometry":2.0,"lat":2,"lon":2.0},{"fill_colour":[53.0,183.0,121.0,255.0],"geometry":-5.0,"lat":3,"lon":-5.0},{"fill_colour":[253.0,231.0,37.0,255.0],"geometry":0.3,"lat":4,"lon":0.3}]')
+
+  
+})
+
+test_that("issue 55 is solved",{
+  
+  df <- data.frame(string = c("first", NA))
+  js <- to_json( df )
+  expect_equal( as.character( js ), '[{"string":"first"},{"string":null}]')
+  
+})
+
